"""Exceptions for aioccl."""

class CCLDeviceRegistrationException(Exception):
    """Exception when registering a new device."""

class CCLDataUpdateException(Exception):
    """Exception when updating data."""